@@ -233,7 +233,7 @@ enum kernel_type {
 };
 
 static PyObject *
-two_bitarray_func(PyObject *args, enum kernel_type kern, const char *format)
+binary_function(PyObject *args, enum kernel_type kern, const char *format)
 {
     Py_ssize_t res = 0, nbytes, i;
     PyObject *a, *b;
@@ -298,7 +298,7 @@ two_bitarray_func(PyObject *args, enum kernel_type kern, const char *format)
 static PyObject *                                                       \
 count_ ## oper (bitarrayobject *module, PyObject *args)                 \
 {                                                                       \
-    return two_bitarray_func(args, KERN_c ## oper, "OO:count_" #oper);  \
+    return binary_function(args, KERN_c ## oper, "OO:count_" #oper);    \
 }                                                                       \
 PyDoc_STRVAR(count_ ## oper ## _doc,                                    \
 "count_" #oper "(a, b, /) -> int\n\
@@ -306,15 +306,15 @@ PyDoc_STRVAR(count_ ## oper ## _doc,                                    \
 Return `(a " ochar " b).count()` in a memory efficient manner,\n\
 as no intermediate bitarray object gets created.")
 
-COUNT_FUNC(and, "&");
-COUNT_FUNC(or,  "|");
-COUNT_FUNC(xor, "^");
+COUNT_FUNC(and, "&");           /* count_and */
+COUNT_FUNC(or,  "|");           /* count_or  */
+COUNT_FUNC(xor, "^");           /* count_xor */
 
 
 static PyObject *
 subset(PyObject *module, PyObject *args)
 {
-    return two_bitarray_func(args, KERN_subset, "OO:subset");
+    return binary_function(args, KERN_subset, "OO:subset");
 }
 
 PyDoc_STRVAR(subset_doc,
@@ -466,6 +466,160 @@ hex2ba(PyObject *module, PyObject *args)
     Py_RETURN_NONE;
 }
 
+/* ----------------------- base 2, 4, 8, 16, 32, 64 -------------------- */
+
+/* RFC 4648 Base32 alphabet */
+static const char base32_alphabet[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ234567";
+
+/* standard base 64 alphabet */
+static const char base64_alphabet[] =
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
+
+static int
+digit_to_int(char c, int n)
+{
+    if (n <= 16) {              /* base 2, 4, 8, 16 */
+        int i;
+
+        i  = hex_to_int(c);
+        return (0 <= i && i < n) ? i : -1;
+    }
+    if (n == 32) {              /* base 32 */
+        if ('A' <= c && c <= 'Z')
+            return c - 'A';
+        if ('2' <= c && c <= '7')
+            return c - '2' + 26;
+    }
+    if (n == 64) {              /* base 64 */
+        if ('A' <= c && c <= 'Z')
+            return c - 'A';
+        if ('a' <= c && c <= 'z')
+            return c - 'a' + 26;
+        if ('0' <= c && c <= '9')
+            return c - '0' + 52;
+        if (c == '+')
+            return 62;
+        if (c == '/')
+            return 63;
+    }
+    return -1;
+}
+
+static int
+base_to_length(int n)
+{
+    int k;
+
+    for (k = 1; k < 7; k++) {
+        if (n == (1 << k))
+            return k;
+    }
+    PyErr_SetString(PyExc_ValueError, "base must be 2, 4, 8, 16, 32 or 64");
+    return -1;
+}
+
+static PyObject *
+ba2base(PyObject *module, PyObject *args)
+{
+    const char *alphabet;
+    PyObject *result, *a;
+    size_t i, strsize;
+    char *str;
+    int n, m, x, k, le;
+
+    if (!PyArg_ParseTuple(args, "iO:ba2ascii", &n, &a))
+        return NULL;
+    if (ensure_bitarray(a) < 0)
+        return NULL;
+    m = base_to_length(n);
+    if (m < 0)
+        return NULL;
+
+    if (n == 32)
+        alphabet = base32_alphabet;
+    else if (n == 64)
+        alphabet = base64_alphabet;
+    else
+        alphabet = hexdigits;
+
+#define aa  ((bitarrayobject *) a)
+    if (aa->nbits % m)
+        return PyErr_Format(PyExc_ValueError,
+                            "bitarray length not multiple of %d", m);
+
+    strsize = aa->nbits / m;
+    str = (char *) PyMem_Malloc(strsize);
+    if (str == NULL)
+        return PyErr_NoMemory();
+
+    le = IS_LE(aa);
+    for (i = 0; i < strsize; i++) {
+        x = 0;
+        for (k = 0; k < m; k++)
+            x |= GETBIT(aa, m * i + (le ? k : (m - k - 1))) << k;
+        str[i] = alphabet[x];
+    }
+    result = Py_BuildValue("s#", str, strsize);
+#undef aa
+    PyMem_Free((void *) str);
+    return result;
+}
+
+PyDoc_STRVAR(ba2base_doc,
+"ba2base(n, bitarray, /) -> str\n\
+\n\
+Return a string containing the base `n` ASCII representation of\n\
+the bitarray.  Allowed values for `n` are 2, 4, 8, 16, 32 and 64.\n\
+The bitarray has to be multiple of length 1, 2, 3, 4, 5 or 6 respectively.\n\
+For `n=16` (hexadecimal), `ba2hex()` will be much faster, as `ba2base()`\n\
+does not take advantage of byte level operations.\n\
+For `n=32` the RFC 4648 Base32 alphabet is used, and for `n=64` the\n\
+standard base 64 alphabet is used.");
+
+
+/* Translate ASCII digits into the bitarray's buffer.
+   The (Python) arguments to this functions are:
+   - the base n, either 2, 4, 8, 16, 32 or 64    (n=2^m - m bits per digits)
+   - bitarray (of length m * len(s)) whose buffer is written into
+   - byte object s containing the ASCII digits
+*/
+static PyObject *
+base2ba(PyObject *module, PyObject *args)
+{
+    PyObject *a;
+    Py_ssize_t i, strsize;
+    char *str;
+    int n, m, d, k, le;
+
+    if (!PyArg_ParseTuple(args, "iOs#", &n, &a, &str, &strsize))
+        return NULL;
+    if (ensure_bitarray(a) < 0)
+        return NULL;
+    m = base_to_length(n);
+    if (m < 0)
+        return NULL;
+
+#define aa  ((bitarrayobject *) a)
+    if (aa->nbits != m * strsize) {
+        PyErr_SetString(PyExc_ValueError, "size mismatch");
+        return NULL;
+    }
+    memset(aa->ob_item, 0x00, (size_t) Py_SIZE(a));
+
+    le = IS_LE(aa);
+    for (i = 0; i < strsize; i++) {
+        d = digit_to_int(str[i], n);
+        if (d < 0) {
+            PyErr_SetString(PyExc_ValueError, "Invalid digit found");
+            return NULL;
+        }
+        for (k = 0; k < m; k++)
+            setbit(aa, m * i + (le ? k : (m - k - 1)), d & (1 << k));
+    }
+#undef aa
+    Py_RETURN_NONE;
+}
+
 /* --------------------------------------------------------------------- */
 
 /* set bitarray_type_obj (bato) */
@@ -487,6 +641,8 @@ static PyMethodDef module_functions[] = {
     {"serialize", (PyCFunction) serialize, METH_O,       serialize_doc},
     {"ba2hex",    (PyCFunction) ba2hex,    METH_O,       ba2hex_doc},
     {"_hex2ba",   (PyCFunction) hex2ba,    METH_VARARGS, 0},
+    {"ba2base",   (PyCFunction) ba2base,   METH_VARARGS, ba2base_doc},
+    {"_base2ba",  (PyCFunction) base2ba,   METH_VARARGS, 0},
     {"_set_bato", (PyCFunction) set_bato,  METH_O,       0},
     {NULL,        NULL}  /* sentinel */
 };
